.titleLogo {
  width: 84px;
}

md-list-item .md-list-item-inner > .md-avatar img,
md-list-item > .md-avatar img {
  border-radius: 50%;
  box-sizing: content-box;
  height: 40px;
  margin-bottom: 8px;
  margin-right: 16px;
  margin-top: 8px;
  width: 40px;
}

.redTab {
  padding: 14px;
  background-color: #F44336;
  color: white;
}
