[Service]
User=root
# Note: ExecStart= is required to clear the ExecStart in the unit file
ExecStart=
ExecStart={{ bin_dir }}/kube-controller-manager \
	    $KUBE_LOGTOSTDERR \
	    $KUBE_LOG_LEVEL \
	    $KUBE_MASTER \
	    $KUBE_CONTROLLER_MANAGER_ARGS
